#include <stdio.h>
#include <bfd.h>
#include <dis-asm.h>
                
int my_disas(unsigned char * buffer, bfd_size_type size, int vma, FILE * ostream)
{
	int minlen=6;
	int bytes;
	disassembler_ftype fun;
	disassemble_info info;
	INIT_DISASSEMBLE_INFO(info, ostream, fprintf);

	info.flavour = bfd_target_unknown_flavour;
	info.arch = bfd_arch_i386;
#ifdef __i386__
	info.mach = bfd_mach_i386_i386;
#elif defined(__x86_64__)
	info.mach = bfd_mach_x86_64;
#else
#error unknown arch		
#endif
	info.endian = BFD_ENDIAN_LITTLE;
	fun = print_insn_i386;

	info.buffer = buffer;
	info.buffer_length = size;
	info.buffer_vma = vma;
	bytes = 0;
	while (bytes < minlen && bytes<size) {
		if (buffer[bytes]==0xeb || (buffer[bytes]&0x70)==0x70) {
			int off=buffer[bytes+1];
			if (off<128 && minlen<bytes+off+2)
				minlen=bytes+off+2;
		}
		fprintf(ostream, "0x%x: ", vma + bytes);
		info.insn_info_valid=0;
		bytes += fun(vma + bytes, &info);
		if (info.insn_info_valid)
		printf(" type %d\n", info.insn_type);
		else printf("\n");
	}
	if (bytes>=size) {
		fprintf(stderr, "out of vma size (too far jcc) ?\n");
		exit(1);
	}
	return bytes;
}
#define CODESIZE 40
main(int argc, char ** argv)
{
	FILE * fin, *fout;
	int ret;
	char buf[CODESIZE];
	if (argc!=3) {
		fprintf(stderr, "usage: disas infile outfile_with_len\n");
		exit(1);
	}
	fin=fopen(argv[1], "r");
	if (!fin) {
		perror("fopen infile");
		exit(1);
	}
	fout=fopen(argv[2], "w");
	if (!fout) {
		perror("fopen outfile");
		exit(1);
	}
	if (fread(buf, 1, CODESIZE, fin)!=CODESIZE) {
		perror("fread");
		exit(1);
	}
	ret=my_disas(buf, CODESIZE,0, stdout);
	printf("size=%d, if there are position dependent instructions, "
	"press ctrl-c to abort compilation, else press return\n", ret);
	getchar();
	fprintf(fout, "%d", ret);
	return 0;
}
